import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

enum InterstitialEvent {
  onLoadAd,
  onShowAd,
  onFailedAd,
  onCloseAd
}

typedef void FlutterInterstitialListener(InterstitialEvent event);

class FlutterBaseInterstitial {
  static const _DEFAULT_CHANNELNAME = "com.bidmad.sdk.interstitial/Flutter";
  String _channelNm;
  MethodChannel _platform;
  FlutterInterstitialListener _listener;

  Map<String, InterstitialEvent> interstitialEventMap = <String, InterstitialEvent>{
    'onLoadAd': InterstitialEvent.onLoadAd,
    'onShowAd': InterstitialEvent.onShowAd,
    'onFailedAd': InterstitialEvent.onFailedAd,
    'onCloseAd': InterstitialEvent.onCloseAd
  };

  String get channelNm => _channelNm;

  FlutterBaseInterstitial({
    @required FlutterInterstitialListener listener,
    String channelName
  }){
    this._listener = listener;
    this._channelNm = channelName ?? _DEFAULT_CHANNELNAME;
    _initChannel(_channelNm);
  }

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
    _platform.setMethodCallHandler(_handleMethod);
  }

  Future<void> load() async {
    await _platform.invokeMethod('load');
  }

  Future<void> show() async {
    await _platform.invokeMethod('show');
  }

  Future<void> setAdInfo(String zoneId) async {
    await _platform.invokeMethod('setAdInfo', zoneId);
  }

  Future<dynamic> _handleMethod(MethodCall call) async {

    switch (call.method) {
      case 'onLoadAd':
      case 'onShowAd':
      case 'onFailedAd':
      case 'onCloseAd':
        final InterstitialEvent interstitialAdEvent = interstitialEventMap[call.method];
        _listener(interstitialAdEvent);
        break;
      default:
        throw ('method not defined');
    }
  }
}