import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';

enum RewardEvent {
  onLoadAd,
  onShowAd,
  onFailedAd,
  onCompleteAd,
  onOpenAd,
  onCloseAd,
  onClickAd,
  onSkippedAd,
}

typedef void FlutterRewardListener(RewardEvent event, String zoneId);

class FlutterBaseReward{
  static const _DEFAULT_CHANNELNAME = "com.bidmad.sdk.reward/Flutter";
  String _channelNm;
  MethodChannel _platform;
  FlutterRewardListener _listener;

  var onLoadAdCallback = null;
  var onShowAdCallback = null;
  var onFailAdCallback = null;
  var onCompleteAdCallback = null;
  var onOpenAdCallback = null;
  var onCloseAdCallback = null;
  var onClickAdCallback = null;
  var onSkippedAdCallback = null;

  String get channelNm => _channelNm;

  FlutterBaseReward({
    @required String channelName
  }){
    print("channelName = " + channelName);
    print("set channelName = " + channelName ?? _DEFAULT_CHANNELNAME);
    this._channelNm = channelName ?? _DEFAULT_CHANNELNAME;
    _initChannel(_channelNm);
  }

  Future<void> _initChannel(String channelNm) async {
    _platform = MethodChannel(channelNm);
    _platform.setMethodCallHandler(_handleMethod);
  }

  Future<void> load() async {
    await _platform.invokeMethod('load');
  }

  Future<void> show() async {
    await _platform.invokeMethod('show');
  }

  Future<void> setAdInfo(String zoneId) async {
    await _platform.invokeMethod('setAdInfo', zoneId);
  }

  Future<void> setCallbackListener({
    void Function(String zoneId) onLoadAd,
    void Function(String zoneId) onShowAd,
    void Function(String zoneId) onFailAd,
    void Function(String zoneId) onCompleteAd,
    void Function(String zoneId) onCloseAd,
    void Function(String zoneId) onClickAd,
    void Function(String zoneId) onSkippedAd
  }) async {

    if(onLoadAd != null){
      onLoadAdCallback = onLoadAd;
    }
    if(onShowAd != null){
      onShowAdCallback = onShowAd;
    }
    if(onFailAd != null){
      onFailAdCallback = onFailAd;
    }
    if(onCompleteAd != null){
      onCompleteAdCallback = onCompleteAd;
    }
    if(onCloseAd != null){
      onCloseAdCallback = onCloseAd;
    }
    if(onClickAd != null){
      onClickAdCallback = onClickAd;
    }
    if(onSkippedAd != null){
      onSkippedAdCallback = onSkippedAd;
    }
  }

  Future<dynamic> _handleMethod(MethodCall call) async {
    String zoneId = call.arguments.toString();
    switch (call.method) {
      case 'onLoadAd':
        if (onLoadAdCallback != null) {
          onLoadAdCallback(zoneId);
        };
        break;
      case 'onShowAd':
        if (onShowAdCallback != null) {
          onShowAdCallback(zoneId);
        };
        break;
      case 'onFailAd':
        if (onFailAdCallback != null) {
          onFailAdCallback(zoneId);
        };
        break;
      case 'onCompleteAd':
        if (onCompleteAdCallback != null) {
          onCompleteAdCallback(zoneId);
        };
        break;
      case 'onOpenAd':
        if (onOpenAdCallback != null) {
          onOpenAdCallback(zoneId);
        };
        break;
      case 'onCloseAd':
        if (onCloseAdCallback != null) {
          onCloseAdCallback(zoneId);
        };
        break;
      case 'onClickAd':
        if (onClickAdCallback != null) {
          onClickAdCallback(zoneId);
        };
        break;
      case 'onSkippedAd':
        if (onSkippedAdCallback != null) {
          onSkippedAdCallback(zoneId);
        };
        break;
      default:
        throw ('method not defined');
    }
  }
}